package rekursion_test

/* ---------------------------------------------------------------- *
 * UNIT TESTING
 * ---------------------------------------------------------------- */

import (
	"logik/aussagenlogik/rekursion"
	"logik/aussagenlogik/schema"
	"logik/aussagenlogik/syntaxbaum"
	"logik/core/utils"
	"testing"

	"github.com/stretchr/testify/assert"
)

/* ---------------------------------------------------------------- *
 * TESTCASE eval(·, ·)
 * ---------------------------------------------------------------- */

func TestRekursivEvalLiteral(test *testing.T) {
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	var I []string
	tree = schema.ParseExpr("A0")
	I = []string{"A0"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 1)
	tree = schema.ParseExpr("A0")
	I = []string{}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 0)
	tree = schema.ParseExpr("! A0")
	I = []string{"A0"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 0)
	tree = schema.ParseExpr("! A0")
	I = []string{}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 1)
}

func TestRekursivEvalComplex1(test *testing.T) {
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	var I []string
	tree = schema.ParseExpr("( ! A0 || (( A0 && A3 ) || A2 ))")
	I = []string{"A0", "A2"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 1)
	I = []string{"A0", "A3"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 1)
	I = []string{"A0"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 0)
	I = []string{"A4", "A8"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 1)
}

func TestRekursivEvalComplex2(test *testing.T) {
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	var I []string
	tree = schema.ParseExpr("( ! A0 || (( A0 && A3 ) || ! A2 ))")
	I = []string{"A0", "A2"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 0)
	I = []string{"A0", "A3"}
	go rekursion.RekursivEval(ch, tree, I)
	assert.Equal(<-ch, 1)
}

/* ---------------------------------------------------------------- *
 * TESTCASE Atoms(·)
 * ---------------------------------------------------------------- */

func TestRekursivAtomsNoduplicates(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan []string)
	var tree syntaxbaum.SyntaxBaum
	var val []string
	tree = schema.ParseExpr("( A4 && ( A4 || A4 ))")
	go rekursion.RekursivAtoms(ch, tree)
	val = <-ch
	var n int = len(utils.FilterStrings(&val, func(x string) bool { return x == "A4" }))
	assert.Equal(n, 1, "Atome dürfen nicht mehrfach vorkommen!")
}

func TestRekursivAtomsNononatoms(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	test.Skip("Syntax for generic expressions in ANTLR4 g4 needs to be implemented.")
	var assert = assert.New(test)
	var ch = make(chan []string)
	var tree syntaxbaum.SyntaxBaum
	var val []string
	tree = schema.ParseExpr("( {F} || A3 )")
	go rekursion.RekursivAtoms(ch, tree)
	val = <-ch
	utils.SortStrings(&val)
	assert.NotContains(val, "F", "Nichtatomare Formeln dürfen nicht vorkommen!")
}

func TestRekursivAtomsCalc1(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan []string)
	var tree syntaxbaum.SyntaxBaum
	var val []string
	tree = schema.ParseExpr("A0")
	go rekursion.RekursivAtoms(ch, tree)
	val = <-ch
	utils.SortStrings(&val)
	assert.Equal(val, []string{"A0"})
}

func TestRekursivAtomsCalc2(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan []string)
	var tree syntaxbaum.SyntaxBaum
	var val []string
	tree = schema.ParseExpr("((( ! A8 && A3 ) || A4 ) && A0 )")
	go rekursion.RekursivAtoms(ch, tree)
	val = <-ch
	utils.SortStrings(&val)
	assert.Equal(val, []string{"A0", "A3", "A4", "A8"})
}

/* ---------------------------------------------------------------- *
 * TESTCASE depth(·, ·)
 * ---------------------------------------------------------------- */

func TestRekursivDepthCalc1(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("A0")
	go rekursion.RekursivDepth(ch, tree)
	assert.Equal(<-ch, 0)
}

func TestRekursivDepthCalc2(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("!! A8")
	go rekursion.RekursivDepth(ch, tree)
	assert.Equal(<-ch, 2)
}

func TestRekursivDepthCalc3(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("( ! A0 && A3 )")
	go rekursion.RekursivDepth(ch, tree)
	assert.Equal(<-ch, 2)
}

func TestRekursivDepthCalc4(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("((( ! A0 && A3 ) || A4 ) && A8 )")
	go rekursion.RekursivDepth(ch, tree)
	assert.Equal(<-ch, 4)
}

func TestRekursivDepthCalc5(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("! ((( ! A0 && A3 ) || A4 ) && A8 )")
	go rekursion.RekursivDepth(ch, tree)
	assert.Equal(<-ch, 5)
}

/* ---------------------------------------------------------------- *
 * TESTCASE length(·)
 * ---------------------------------------------------------------- */

func TestRekursivLengthCalc1(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("A0")
	go rekursion.RekursivLength(ch, tree)
	assert.Equal(<-ch, 1)
}

func TestRekursivLengthCalc2(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("!! A8")
	go rekursion.RekursivLength(ch, tree)
	assert.Equal(<-ch, 3)
}

func TestRekursivLengthCalc3(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("( ! A0 && A3 )")
	go rekursion.RekursivLength(ch, tree)
	assert.Equal(<-ch, 4)
}

func TestRekursivLengthCalc4(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("((( ! A0 && A3 ) || A4 ) && A8 )")
	go rekursion.RekursivLength(ch, tree)
	assert.Equal(<-ch, 8)
}

func TestRekursivLengthCalc5(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("! ((( ! A0 && A3 ) || A4 ) && A8 )")
	go rekursion.RekursivLength(ch, tree)
	assert.Equal(<-ch, 9)
}

/* ---------------------------------------------------------------- *
 * TESTCASE #Parentheses(·)
 * ---------------------------------------------------------------- */

func TestRekursivParenthesesCalc1(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("A0")
	go rekursion.RekursivParentheses(ch, tree)
	assert.Equal(<-ch, 0)
}

func TestRekursivParenthesesCalc2(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("!! A8")
	go rekursion.RekursivParentheses(ch, tree)
	assert.Equal(<-ch, 0)
}

func TestRekursivParenthesesCalc3(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("( ! A0 && A3 )")
	go rekursion.RekursivParentheses(ch, tree)
	assert.Equal(<-ch, 2)
}

func TestRekursivParenthesesCalc4(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("((( ! A0 && A3 ) || A4 ) && A8 )")
	go rekursion.RekursivParentheses(ch, tree)
	assert.Equal(<-ch, 6)
}

func TestRekursivParenthesesCalc5(test *testing.T) {
	test.Skip("Methode noch nicht implementiert")
	var assert = assert.New(test)
	var ch = make(chan int)
	var tree syntaxbaum.SyntaxBaum
	tree = schema.ParseExpr("! ((( ! A0 && A3 ) || A4 ) && A8 )")
	go rekursion.RekursivParentheses(ch, tree)
	assert.Equal(<-ch, 6)
}
